package com.install4j.api;

import com.install4j.api.context.Context;
import com.install4j.api.context.UserCanceledException;
import com.install4j.runtime.installer.InstallerConstants;
import com.install4j.runtime.installer.InstallerVariables;
import com.install4j.runtime.installer.frontend.GUIHelper;
import com.install4j.runtime.installer.helper.InstallerUtil;
import com.install4j.runtime.installer.helper.Logger;
import com.install4j.runtime.installer.helper.comm.ExecutionContext;
import com.install4j.runtime.installer.helper.comm.HelperCommunication;
import com.install4j.runtime.installer.helper.comm.actions.FetchIntAction;
import com.install4j.runtime.installer.helper.comm.actions.RunAction;
import com.install4j.runtime.installer.platform.UserInfo;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;
import java.net.URL;

/**
 * The class contains various utility functions to be used by
 * script properties, custom actions and custom screens.
 *
 * @author ej-technologies GmbH
 */
public class Util {

    /**
     * Returns the home directory. On Windows 2000/XP, the location of the <i>My documents</i> folder
     * will be returned, rather than the value of <tt>System.getProperty("java.home")</tt>
     * @return the home directory
     */
    public static String getUserHome() {
        String userHome = InstallerVariables.getStringVariable(InstallerVariables.VARIABLE_USER_HOME);
        if (userHome == null) userHome = System.getProperty("user.home");
        return userHome;
    }

    /**
     * Returns whether the platform is Solaris.
     *
     * @return the result
     */
    public static boolean isSolaris() {
        return InstallerUtil.isSolaris();
    }

    /**
     * Returns whether the platform is Linux.
     *
     * @return the result
     */
    public static boolean isLinux() {
        return InstallerUtil.isLinux();
    }

    /**
     * Returns whether the platform is Windows.
     *
     * @return the result
     */
    public static boolean isWindows() {
        return InstallerUtil.isWindows();
    }

    /**
     * Returns whether the platform is Windows 9X (95, 98, ME).
     *
     * @return the result
     * @deprecated always returns <tt>false</tt> because launchers do not work with Windows 9X anymore
     */
    public static boolean isWindows9X() {
        return false;
    }

    /**
     * Returns whether the platform is Windows NT (not Windows 2000 and XP).
     *
     * @return the result
     */
    public static boolean isWindowsNT() {
        return InstallerUtil.isWindowsNT();
    }

    /**
     * Returns whether the platform is at least Windows Vista.
     *
     * @return the result
     */
    public static boolean isAtLeastWindowsVista() {
        return InstallerUtil.isAtLeastWindowsVista();
    }

    /**
     * Returns whether the platform is at least Windows 7.
     *
     * @return the result
     */
    public static boolean isAtLeastWindows7() {
        return InstallerUtil.isAtLeastWindows7();
    }

    /**
     * Returns whether the platform is at least Windows XP.
     *
     * @return the result
     */
    public static boolean isAtLeastWindowsXP() {
        return InstallerUtil.isAtLeastWindowsXP();
    }

    /**
     * Returns whether the platform is Windows Vista.
     *
     * @return the result
     */
    public static boolean isWindowsVista() {
        return InstallerUtil.isWindowsVista();
    }

    /**
     * Returns whether the platform is Windows 7.
     *
     * @return the result
     */
    public static boolean isWindows7() {
        return InstallerUtil.isWindows7();
    }

    /**
     * Returns whether the platform is Windows 8.
     *
     * @return the result
     */
    public static boolean isWindows8() {
        return InstallerUtil.isWindows8();
    }

    /**
     * Returns whether the platform is Windows XP.
     *
     * @return the result
     */
    public static boolean isWindowsXP() {
        return InstallerUtil.isWindowsXP();
    }

    /**
     * Returns whether the platform is Windows 2000.
     *
     * @return the result
     */
    public static boolean isWindows2000() {
        return InstallerUtil.isWindows2000();
    }

    /**
     * Returns whether the platform is Windows 2003.
     *
     * @return the result
     */
    public static boolean isWindows2003() {
        return InstallerUtil.isWindows2003();
    }

    /**
     * Returns whether the platform is Windows 2008.
     *
     * @return the result
     */
    public static boolean isWindows2008() {
        return InstallerUtil.isWindows2008();
    }

    /**
     * Returns whether the platform is Windows 2012.
     *
     * @return the result
     */
    public static boolean isWindows2012() {
        return InstallerUtil.isWindows2012();
    }

    /**
     * Returns whether the Windows is a 64-bit Windows, regardless of whether the installer is running with a 32-bit JVM or
     * a 64-bit JVM. This condition cannot be found out by inspecting <tt>System.getProperty("os.arch")</tt>, since this will
     * return a value that corresponds to the JRE and not to the OS (i.e. always "x86" for a 32-bit JRE, even if it is running
     * on a 64-bit Windows).
     * <p>
     * This method inspects the environment variables <tt>PROCESSOR_ARCHITECTURE</tt> and <tt>PROCESSOR_ARCHITEW6432</tt>.
     * If <tt>PROCESSOR_ARCHITECTURE</tt> contains <tt>AMD64</tt> or <tt>IA64</tt>, the currently used JRE is a 64-bit JRE which is then
     * by definition running on a 64-bit Windows. If <tt>PROCESSOR_ARCHITECTURE</tt> contains <tt>x86</tt>, the <tt>PROCESSOR_ARCHITEW6432</tt>
     * is undefined for a 32-bit Windows and contains <tt>AMD64</tt> or <tt>IA64</tt> for a 64-bit Windows.
     * @return the result. If the current OS is not Windows, the result is always <tt>false</tt>.
     */
    public static boolean is64BitWindows() {
        return isWindows() && InstallerUtil.is64BitWindows();
    }

    /**
     * Returns whether the platform is Mac OS X.
     *
     * @return the result
     */
    public static boolean isMacOS() {
        return InstallerUtil.isMacOS();
    }

    /**
     * Returns whether the platform is HP UX.
     *
     * @return the result
     */
    public static boolean isHpux() {
        return InstallerUtil.isHpux();
    }

    /**
     * Returns whether the platform is AIX.
     *
     * @return the result
     */
    public static boolean isAix() {
        return InstallerUtil.isAix();
    }

    /**
     * Returns whether the installer is a Windows installer
     *
     * @return the result
     */
    public static boolean isWindowsInstaller() {
        return InstallerUtil.getInstallerType().equals(InstallerConstants.TYPE_WINDOWS);
    }

    /**
     * Returns whether the installer is a Unix installer
     *
     * @return the result
     */
    public static boolean isUnixInstaller() {
        return InstallerUtil.getInstallerType().equals(InstallerConstants.TYPE_UNIX);
    }

    /**
     * Returns whether the installer is a Mac OS X installer
     *
     * @return the result
     */
    public static boolean isMacosInstaller() {
        return InstallerUtil.getInstallerType().equals(InstallerConstants.TYPE_MACOS);
    }

    /**
     * Returns the standard directory for installing applications.
     *
     * @return the directory
     */
    public static String getStandardApplicationsDirectory() {
        return InstallerUtil.getStandardApplicationsDirectory();
    }

    /**
     * Show a URL in the default browser. If no default browser can be
     * determined for the operating system, the user is asked to locate the
     * executable of the internet browser. This executable is cached so that
     * subsequent invocations of this method do not bring up this questions again.
     * <p>
     * For quiet installers, this method does nothing.
     * </p>
     *
     * @param url the URL to be shown in the browser.
     */
    public static void showUrl(final URL url) {
        HelperCommunication.getInstance().executeAction(ExecutionContext.UNELEVATED, new RunAction() {
            @Override
            protected void run(Context context) {
                GUIHelper.showURL(url.toExternalForm());
            }
        });
    }

    /**
     * Show a message dialog. This message dialog is a JOptionPane, the parameters are
     * equivalent to those of the standard Java class. This method works in both GUI and console mode.
     *
     * @param message     the message
     * @param messageType the message type (see JOptionPane)
     * @see javax.swing.JOptionPane
     */
    public static void showMessage(final String message, final int messageType) {
        HelperCommunication.getInstance().executeAction(ExecutionContext.UNELEVATED, new RunAction() {
            @Override
            protected void run(Context context) {
                GUIHelper.showMessage(null, message, messageType);
            }
        });
    }

    /**
     * Show a message dialog. This message dialog is a JOptionPane with a message type of INFORMATION_MESSAGE.
     * This method works in both GUI and console mode.
     *
     * @param message the message
     * @see javax.swing.JOptionPane
     */
    public static void showMessage(String message) {
        showMessage(message, JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Show a message dialog. This message dialog is a JOptionPane with a message type of ERROR_MESSAGE.
     * This method works in both GUI and console mode.
     *
     * @param message the message
     * @see javax.swing.JOptionPane
     */
    public static void showErrorMessage(String message) {
        showMessage(message, JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Show a message dialog. This message dialog is a JOptionPane with a message type of WARNING_MESSAGE.
     * This method works in both GUI and console mode.
     *
     * @param message the message
     * @see javax.swing.JOptionPane
     */
    public static void showWarningMessage(String message) {
        showMessage(message, JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Show an options dialog. This option dialog is a JOptionPane, the parameters are
     * equivalent to those of the standard Java class. This method works in both GUI and console mode.
     *
     * @param message     the message
     * @param options     the options. To explicitly set keys for answers in console mode, prefix the desired letter
     *                    with '&'. For example <tt>new String[] {"&amp;Enter", "E&amp;xit"}</tt>. If no explicit keys are set
     *                    the first letters of the options are taken. Repeating letters are replaced with numeric indices.
     * @param messageType the message type (see JOptionPane)
     * @return the index of the selected option or -1 if the installer is running in unattended mode
     * @throws com.install4j.api.context.UserCanceledException
     *          if the user cancels the selection in console mode
     * @see javax.swing.JOptionPane
     */
    public static int showOptionDialog(final String message, final String[] options, final int messageType) throws UserCanceledException {
        try {
            return HelperCommunication.getInstance().fetchIntChecked(ExecutionContext.UNELEVATED, new FetchIntAction() {
                @Override
                protected int fetchValue(Context context) throws UserCanceledException {
                    return GUIHelper.showOptionDialog(null, message, options, messageType);
                }
            });
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * This method can be invoked if you encounter a non-recoverable and non-expected error in
     * your custom code. The exception and all system properties will be printed to
     * a log file and the user will be informed with a crash dialog. The installer
     * will then quit.
     *
     * @param t the exception that occurred
     */
    public static void fatalError(final Throwable t) {
        try {
            HelperCommunication.getInstance().executeAction(ExecutionContext.UNELEVATED, new RunAction() {
                @Override
                protected void run(Context context) throws Exception {
                    InstallerUtil.reportException(t);
                    InstallerUtil.exit(1);
                }
            });
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    /**
     * Get the window of the installer.
     *
     * @return the window or <tt>null</tt> if the installer is silent
     */
    public static Window getParentWindow() {
        return GUIHelper.getParentWindow();
    }

    /**
     * Writes an info message to the installation log.
     *
     * @param source the object that requests the log. Usually the bean that executes this code. Can be <tt>null</tt>, in this
     * case the currently executed action or - if no action is being executed - the currently displayed screen are used. A
     * case where you should explicitly set this parameter is in code that is executed from form components.
     * @param message the message to be logged.
     */
    public static void logInfo(Object source, String message) {
        Logger.getInstance().info(source, message);
    }

    /**
     * Writes an error message to the installation log.
     *
     * @param source the object that requests the log. Usually the bean that executes this code. Can be <tt>null</tt>, in this
     * case the currently executed action or - if no action is being executed - the currently displayed screen are used. A
     * case where you should explicitly set this parameter is in code that is executed from form components.
     * @param message the message to be logged.
     */
    public static void logError(Object source, String message) {
        Logger.getInstance().error(source, message);
    }

    /**
     * Writes an stack trace to the installation log.
     *
     * @param t the throwable instance
     */
    public static void log(Throwable t) {
        Logger.getInstance().log(t);
    }

    /**
     * Dump all defined installer variables to stderr.
     * In order to be able to see the output you have to create a debug installer by selecting
     * the corresponding option in the Build step in the install4j GUI.
     *
     * @param context the context
     */
    public static void dumpVariables(Context context) {
        System.err.println();
        System.err.println("Defined installer variables:");
        System.err.println("----------------------------");
        for (String variableName : context.getVariableNames()) {
            Object value = context.getVariable(variableName);
            System.err.println(InstallerUtil.getVerboseVariableDefinition(variableName, value));
        }
        System.err.println("----------------------------");
        System.err.println();
    }

    /**
     * Checks whether the current user has administration privileges.
     * <ul>
     * <li>On Windows NT/2000/XP this is the case, if the user is a member of the "Administrators" group.</li>
     * <li>On Windows Vista this method returns <tt>true</tt> if the installer is running with administration privileges. This can be
     * specified by the user when starting the installer. Alternatively, you can request that the installers on Windows Vista always runs
     * with administration privileges in the Windows media wizard in the install4j GUI.</li>
     * <li>On Mac OS X this method returns <tt>true</tt> if the  user is member of the "admin" group.</li>
     * <li>On Unix this method returns <tt>true</tt> if the current user is called "root".</li>
     * </ul>
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     * @deprecated use {@link #isAdminGroup} and {@link #hasFullAdminRights} instead. 
     */
    public static boolean isAdminUser() {
        return UserInfo.isAdminUser();
    }

    /**
     * Checks whether the current user is from the administration group.
     * <ul>
     * <li>On Windows NT and higher this is the case, if the user is a member of the "Administrators" group.</li>
     * <li>On Mac OS X this method returns <tt>true</tt> if the user is member of the "admin" group.</li>
     * <li>On Unix this method returns <tt>true</tt> if the current user is called "root".</li>
     * </ul>
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    public static boolean isAdminGroup() {
        return UserInfo.isAdminGroup();
    }

    /**
     * Checks whether the current process has full administration rights.
     * <ul>
     * <li>On Windows NT/2000/XP this is the case, if the user is a member of the "Administrators" group.</li>
     * <li>On Windows Vista this method returns <tt>true</tt> if the installer is running with elevated privileges.</li>
     * <li>On Mac OS X and Unix this method returns <tt>true</tt> if the current user is called "root".</li>
     * </ul>
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    public static boolean hasFullAdminRights() {
        return UserInfo.hasFullAdminRights();
    }

    /**
     * If a stack trace comes from scripts, it can be difficult to trace them to their origin by looking at the
     * stack trace alone. With this method you can annotate the stack trace with the actual names of properties,
     * actions, screens or form components that contain the offending script code.
     *
     * @param t the exception
     * @return the annotated stack trace
     */
    public static String getAnnotatedStackTrace(Throwable t) {
        return InstallerUtil.getAnnotatedStackTrace(t);
    }

    /**
     * Prints the return value of {@link #getAnnotatedStackTrace(Throwable)} to <tt>System.err</tt>.
     * This can be used for debugging purposes.
     *
     * @param t the exception
     */
    public static void printAnnotatedStackTrace(Throwable t) {
        System.err.print(getAnnotatedStackTrace(t));
    }

    /**
     * Show the specified directory or file. On Windows this will open an explorer window and on Linux/Unix it will open a terminal.
     * On OS X it will open a finder window for regular files or directories. It will mount a DMG file and show its content.
     * @param path the directory that should be displayed
     */
    public static void showPath(final String path) {
        HelperCommunication.getInstance().executeAction(ExecutionContext.UNELEVATED, new RunAction() {
            @Override
            protected void run(Context context) {
                GUIHelper.showPath(path, true);
            }
        });
    }

}
